/*****************************************************************************
 * Copyright (c) 2014, 2016 CEA LIST, Christian W. Damus, and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  CEA LIST - Initial API and implementation
 *  Christian W. Damus - bug 493869
 *   
 *****************************************************************************/

package org.eclipse.papyrusrt.umlrt.core.internal.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.papyrus.infra.viewpoints.policy.ViewPrototype;
import org.eclipse.papyrus.uml.diagram.statemachine.CreateStateMachineDiagramCommand;

/**
 * Specific implementation for the creation of the diagram, to avoid setting a name using a popup
 */
public class CreateHeadlessStateMachineDiagramCommand extends CreateStateMachineDiagramCommand {

	@Override
	protected Diagram doCreateDiagram(Resource diagramResource, EObject owner, EObject element, ViewPrototype prototype, String name) {
		HeadlessDiagramCreationHelper helper = new HeadlessDiagramCreationHelper();

		Diagram result = super.doCreateDiagram(diagramResource, owner, element, prototype, name);

		if (result != null) {
			helper.recordOwnerAndElement(result, owner, element);
		}

		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected CommandResult doEditDiagramName(ViewPrototype prototype, String name) {
		// overrides dialog creation to edit the name
		return CommandResult.newOKCommandResult(name);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void initializeDiagram(EObject diagram) {
		super.initializeDiagram(diagram);

		// enforce name = null
		((Diagram) diagram).setName(null);
	}

}