/*****************************************************************************
 * Copyright (c) 2014, 2016 CEA LIST, Christian W. Damus, and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  CEA LIST - Initial API and implementation
 *  Christian W. Damus - bug 493866
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.types.advice;

import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.papyrus.uml.service.types.element.UMLElementTypes;
import org.eclipse.papyrusrt.umlrt.core.types.ElementTypeUtils;
import org.eclipse.uml2.uml.State;

/**
 * Edit Helper Advice for State machines
 */
public class RTStateEditHelperAdvice extends AbstractEditHelperAdvice {

	/**
	 * Dispatches to specific methods, or super otherwise.
	 */
	@Override
	public boolean approveRequest(IEditCommandRequest request) {
		Boolean result = null;

		if (request instanceof CreateElementRequest) {
			result = approveCreateElementRequest((CreateElementRequest) request);
		}

		if (result == null) {
			result = super.approveRequest(request);
		}

		return result;
	}

	/**
	 * Disallow creation of another region in a composite state: composite
	 * states have exactly one region.
	 * 
	 * @param request
	 *            an element creation request
	 * 
	 * @return approval status if the {@code request} is creating a region in a state;
	 *         {@code null} to delegate to the superclass approval, otherwise
	 */
	private Boolean approveCreateElementRequest(CreateElementRequest request) {
		Boolean result = null;

		IElementType typeToCreate = request.getElementType();
		if (ElementTypeUtils.isTypeCompatible(typeToCreate, UMLElementTypes.REGION)) {
			// Creating some kind of region. In a state?
			if (request.getContainer() instanceof State) {
				State stateToEdit = (State) request.getContainer();
				result = !stateToEdit.isComposite();
			}
		}

		return result;
	}
}
